###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" Delete Volume Command for rdmc """

from six.moves import input

from redfish.ris import InstanceNotFoundError

try:
    from rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ReturnCodes,
    )


class DeleteVolumeCommand:
    """Delete volume command"""

    def __init__(self):
        self.ident = {
            "name": "deletevolume",
            "usage": None,
            "description": "To delete a volume "
            "to a controller by index.\n\texample: deletevolume "
            "1 --controller=1\n\n\tTo delete multiple volumes by "
            "index.\n\tExample: deletevolume 1,2 --controller=1"
            "\n\n\tTo delete all volumes on a controller.\n\t"
            "deletevolume --controller=1 --all\n\n\t"
            'deletevolume --controller="Slot1" --all\n\n\tNOTE: '
            "You can also delete volumes by "
            '"VolumeUniqueIdentifier".\n\n\t'
            "For iLO6, storage id need to be specified using --storageid=DE00E000 along with --controller=1\n\n\t"
            "You can also delete volumes by VolumeName",
            "summary": "Deletes volumes from the selected controller.",
            "aliases": ["deletelogicaldrive"],
            "auxcommands": ["SelectCommand", "StorageControllerCommand"],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main disk inventory worker function

        :param line: command line input
        :type line: string.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.deletevolumevalidation(options)
        ilo_ver = self.rdmc.app.getiloversion()
        if ilo_ver < 6.110:
            try:
                self.auxcommands["select"].selectfunction("StorageController.")
                content = self.rdmc.app.getprops()
                ilo_ver = 6.110
            except Exception:
                pass

        if ilo_ver >= 6.110:
            if not options.storageid:
                raise InvalidCommandLineError(
                    "--storageid option is mandatory for iLO6 along with --controller option.\n"
                )
            controller_logicaldrives = self.auxcommands["storagecontroller"].storagelogical_drives(
                options, options.controller, options.storageid, single_use=True
            )
            if len(controller_logicaldrives) == 0:
                raise InvalidCommandLineError("No Logical drives found.\n")
            get_contrller = []
            self.auxcommands["select"].selectfunction("StorageCollection.")
            st_content = self.rdmc.app.getprops()
            for st_controller in st_content:
                path = st_controller["Members"]
                for mem in path:
                    for val in mem.values():
                        if "DE" in val and options.storageid in val:
                            getval = self.rdmc.app.get_handler(val, silent=True, service=True).dict
                            controller = getval["Controllers"]
                            for ctl in controller.values():
                                ctl = ctl + "/0"
                                get_sel = self.rdmc.app.get_handler(ctl, silent=True, service=True)
                            get_contrller.append(get_sel)
        else:
            self.auxcommands["select"].selectfunction("SmartStorageConfig.")
            content = self.rdmc.app.getprops()

        if not args and not options.all:
            raise InvalidCommandLineError("You must include a logical drive to delete.")
        elif not options.controller:
            raise InvalidCommandLineError("You must include a controller to select.")
        else:
            if len(args) > 1:
                logicaldrives = args
            elif len(args) == 1:
                logicaldrives = args[0].replace(", ", ",").split(",")
            else:
                logicaldrives = None

        controllist = []

        try:
            if ilo_ver >= 6.110:
                if options.controller.isdigit():
                    slotlocation = self.storageget_location_from_id(options.controller, options.storageid)
                    if slotlocation:
                        slotcontrol = slotlocation.lower().strip('"').split("slot")[-1].lstrip().strip("=")
                        for control in get_contrller:
                            cont_temp = control.dict
                            temp = cont_temp["Location"]["PartLocation"]["ServiceLabel"]
                            if "Location" in cont_temp and slotcontrol.lower() == temp.lower().split("slot")[
                                -1
                            ].lstrip().strip("="):
                                controllist.append(cont_temp)
                elif "slot" in options.controller.lower():
                    slotlocation = options.controller
                    if slotlocation:
                        slotcontrol = slotlocation.lower().strip('"').split("slot")[-1].lstrip().strip("=")
                        for control in get_contrller:
                            cont_temp = control.dict
                            temp = cont_temp["Location"]["PartLocation"]["ServiceLabel"]
                            if "Location" in cont_temp and slotcontrol.lower() == temp.lower().split("slot")[
                                -1
                            ].lstrip().strip("="):
                                controllist.append(cont_temp)
                if not controllist:
                    raise InvalidCommandLineError("")
            else:
                try:
                    controllers = self.auxcommands["storagecontroller"].storagecontroller(options, single_use=True)
                    options.command = "volume"
                    ilo_ver = 6.110
                except InstanceNotFoundError:
                    if not options.storageid:
                        raise InvalidCommandLineError(
                            "--storageid option is mandatory. Please input storageid as well so that "
                            "controllers/volumes can be identified.\n"
                        )
                    controllers = self.auxcommands["storagecontroller"].storagecontroller(options, single_use=True)
                    options.command = "volume"
                    ilo_ver = 6.110
                if options.controller.isdigit():
                    slotlocation = self.get_location_from_id(options.controller)
                    if slotlocation:
                        slotcontrol = slotlocation.lower().strip('"').split("slot")[-1].lstrip()
                        for control in content:
                            if slotcontrol.lower() == control["Location"].lower().split("slot")[-1].lstrip():
                                controllist.append(control)
                if not controllist:
                    raise InvalidCommandLineError("")
        except InvalidCommandLineError:
            raise InvalidCommandLineError("Selected controller not found in the current inventory " "list.")
        if ilo_ver >= 6.110:
            self.storagedeletevolume(controller_logicaldrives, logicaldrives, options.all, options.force)
        else:
            self.deletevolume(controllist, logicaldrives, options.all, options.force)
            self.rdmc.ui.printer(
                "One or more properties were changed and will not take effect " "until system is reset \n"
            )

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def get_location_from_id(self, controller_id):
        # self.rdmc.ui.printer("Controller ID: %s\n" % (controller_id))
        for sel in self.rdmc.app.select("SmartStorageArrayController", path_refresh=True):
            if "Collection" not in sel.maj_type:
                controller = sel.dict
                self.rdmc.ui.printer("Controller ID: %s\n" % (controller["Id"]))
                if controller["Id"] == str(controller_id):
                    self.rdmc.ui.printer("Controller Location: %s\n" % (controller["Location"]))
                    return controller["Location"]
        return None

    def storageget_location_from_id(self, controller_id, storageid):
        get_contrller = []
        # self.auxcommands["select"].selectfunction("StorageCollection.")
        st_content = self.rdmc.app.getprops()
        for st_controller in st_content:
            path = st_controller["Members"]
            for mem in path:
                for val in mem.values():
                    if "DE" in val and storageid in val:
                        getval = self.rdmc.app.get_handler(val, silent=True, service=True).dict
                        controller = getval["Controllers"]
                        for ctl in controller.values():
                            ctl = ctl + "/0"
                            get_sel = self.rdmc.app.get_handler(ctl, silent=True, service=True)
                        get_contrller.append(get_sel)
        for sel in get_contrller:
            if "Collection" not in sel.path:
                controller = sel.dict
                if controller["Id"] == str(controller_id):
                    return controller["Location"]["PartLocation"]["ServiceLabel"]
        return None

    def deletevolume(self, controllist, drivelist, allopt, force):
        """Gets volumes ready for deletion

        :param controllist: list of controllers
        :type controllist: list.
        :param drivelist: volumes to delete
        :type drivelist: list.
        :param allopt: flag for deleting all volumes
        :type allopt: bool.
        """

        for controller in controllist:
            changes = False

            numdrives = len(controller["LogicalDrives"])
            deletecount = 0

            if allopt:
                controller["LogicalDrives"] = []
                controller["DataGuard"] = "Disabled"
                if not force:
                    if not self.inputaccept(None):
                        return
                self.lastlogicaldrive(controller)
                changes = True
            else:
                for deldrive in drivelist:
                    found = False

                    if deldrive.isdigit():
                        deldrive = int(deldrive)

                    for idx, ldrive in enumerate(controller["LogicalDrives"]):
                        if deldrive == ldrive["LogicalDriveNumber"]:
                            if not force:
                                if not self.inputaccept(ldrive["LogicalDriveName"]):
                                    return

                            controller["LogicalDrives"][idx]["Actions"] = [{"Action": "LogicalDriveDelete"}]

                            controller["DataGuard"] = "Permissive"
                            deletecount += 1

                            changes = True
                            found = True

                            break

                    if not found:
                        raise NoContentsFoundForOperationError("Logical " "drive %s not found." % str(deldrive))

                if deletecount == numdrives:
                    self.lastlogicaldrive(controller)

            if changes:
                # self.rdmc.ui.printer(
                #    "DeleteVolume path and payload: %s, %s\n"
                #    % (controller["@odata.id"], controller)
                # )
                put_path = controller["@odata.id"]
                if "settings" not in put_path:
                    put_path = put_path + "settings/"
                controller["@odata.id"] = put_path
                self.rdmc.app.put_handler(
                    put_path,
                    controller,
                    headers={"If-Match": self.getetag(controller["@odata.id"])},
                )
                self.rdmc.app.download_path([controller["@odata.id"]], path_refresh=True, crawl=False)

    def lastlogicaldrive(self, controller):
        """Special case that sets required properties after last drive deletion

        :param controller: controller change settings on
        :type controller: dict.
        """
        changelist = [
            "PredictiveSpareRebuild",
            "SurfaceScanAnalysisPriority",
            "FlexibleLatencySchedulerSetting",
            "DegradedPerformanceOptimization",
            "CurrentParallelSurfaceScanCount",
            "SurfaceScanAnalysisDelaySeconds",
            "MonitorAndPerformanceAnalysisDelaySeconds",
            "InconsistencyRepairPolicy",
            "DriveWriteCache",
            "ExpandPriority",
            "EncryptionEULA",
            "NoBatteryWriteCache",
            "ReadCachePercent",
            "WriteCacheBypassThresholdKiB",
            "RebuildPriority",
            "QueueDepth",
            "ElevatorSort",
        ]

        for item in changelist:
            if item in list(controller.keys()):
                controller[item] = None

    def inputaccept(self, drivename):
        while True:
            if drivename is None:
                ans = input("Are you sure you would" " like to continue deleting all volumes" "? (y/n)")
            else:
                ans = input("Are you sure you would" " like to continue deleting volume" " %s? (y/n)" % drivename)

            if ans.lower() == "y":
                return 1
            elif ans.lower() == "n":
                self.rdmc.ui.printer("Stopping command without deleting volume/volumes.\n")
                return 0

    def storagedeletevolume(self, logical_drives, drivelist, allopt, force):
        if allopt:
            sorted_dict = sorted(logical_drives, reverse=True)
            if not force:
                if not self.inputaccept(None):
                    return
            for drive in sorted_dict:
                self.rdmc.app.delete_handler(logical_drives[drive]["@odata.id"], {})
        else:
            found = False
            sorted_dict = sorted(logical_drives, reverse=True)
            sorted_drivelist = sorted(drivelist, reverse=True)
            for deldrive in sorted_drivelist:
                found = False
                changes = False
                for drive in sorted_dict:
                    if logical_drives[drive]["Id"] == deldrive:
                        if not force:
                            if not self.inputaccept(logical_drives[drive]["Name"]):
                                return
                        self.rdmc.ui.printer("Setting volume %s " "for deletion\n" % logical_drives[drive]["Name"])
                        changes = True
                        found = True
                        break
                try:
                    if changes:
                        # self.rdmc.ui.printer(
                        #    "DeleteVolume path and payload: %s, %s\n"
                        #    % (logical_drives[drive]["@odata.id"], "{}")
                        # )
                        self.rdmc.app.delete_handler(logical_drives[drive]["@odata.id"], {})
                except:
                    self.rdmc.ui.printer(
                        "This volume cannot"
                        "be deleted without deleting "
                        "the higher volumes in the same Physical drive first.\n"
                    )
                    raise
            if not found:
                raise NoContentsFoundForOperationError("Logical " "drive not found.")

    def getetag(self, path):
        """get etag from path"""
        etag = None
        instance = self.rdmc.app.monolith.path(path)
        if instance:
            etag = (
                instance.resp.getheader("etag")
                if "etag" in instance.resp.getheaders()
                else instance.resp.getheader("ETag")
            )
        return etag

    def deletevolumevalidation(self, options):
        """delete volume validation function

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "--controller",
            dest="controller",
            help="Use this flag to select the corresponding controller " "using either the slot number or index.",
            default=None,
        )
        customparser.add_argument(
            "--storageid",
            dest="storageid",
            help="Use this flag to select the corresponding storageid in iLO6 only.",
            default=None,
        )
        customparser.add_argument(
            "--all",
            dest="all",
            help="""Use this flag to delete all volumes on a controller.""",
            action="store_true",
            default=False,
        )
        customparser.add_argument(
            "--force",
            dest="force",
            help="""Use this flag to override the "are you sure?" text when deleting a volume.""",
            action="store_true",
            default=False,
        )



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" RawDelete Command for rdmc """

import sys

try:
    from rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )


class RawDeleteCommand:
    """Raw form of the delete command"""

    def __init__(self):
        self.ident = {
            "name": "rawdelete",
            "usage": None,
            "description": "Run to to delete data from"
            ' the passed in path.\n\texample: rawdelete "/redfish/v1/'
            'Sessions/(session ID)"\n',
            "summary": "Raw form of the DELETE command.",
            "aliases": [],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main raw delete worker function

        :param line: command line input
        :type line: string.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        headers = {}

        if hasattr(options, "sessionid") and options.sessionid:
            self.sessionvalidation(options)
        else:
            self.deletevalidation(options)

        if options.path.startswith('"') and options.path.endswith('"'):
            options.path = options.path[1:-1]

        if options.expand:
            options.path = options.path + "?$expand=."

        try:
            currentsess = self.rdmc.app.current_client.session_location
        except:
            currentsess = None

        if options.headers:
            extraheaders = options.headers.split(",")

            for item in extraheaders:
                header = item.split(":")

                try:
                    headers[header[0]] = header[1]
                except:
                    InvalidCommandLineError("Invalid format for --headers option.")

        if currentsess and (options.path in currentsess):
            self.rdmc.app.logout()
            self.rdmc.ui.printer(
                "Your session has been terminated (deleted).\nPlease log " "back in if you wish to continue.\n"
            )
        else:
            returnresponse = False

            if options.response or options.getheaders:
                returnresponse = True

            results = self.rdmc.app.delete_handler(
                options.path,
                headers=headers,
                silent=options.silent,
                service=options.service,
            )

            if returnresponse and results:
                if options.getheaders:
                    self.rdmc.ui.print_out_json(dict(results.getheaders()))

                if options.response:
                    sys.stdout.write(results.read)
                    self.rdmc.ui.printer("\n")
            elif results.status == 404:
                return ReturnCodes.NO_CONTENTS_FOUND_FOR_OPERATION
            elif results.status != 200:
                return ReturnCodes.UI_CLI_USAGE_EXCEPTION

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def deletevalidation(self, options):
        """Raw delete validation function

        :param options: command line options
        :type options: list.
        """
        self.rdmc.login_select_validation(self, options, skipbuild=True)

    def sessionvalidation(self, options):
        """Raw delete session validation function

        :param options: command line options
        :type options: list.
        """

        url = None
        if options.user or options.password or options.url:
            if options.url:
                url = options.url
        else:
            if self.rdmc.app.config.get_url():
                url = self.rdmc.app.config.get_url()
        if url and "https://" not in url:
            url = "https://" + url

        return url

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "path",
            help="Uri on iLO to be deleted.",
        )
        customparser.add_argument(
            "--response",
            dest="response",
            action="store_true",
            help="Use this flag to return the iLO response body.",
            default=False,
        )
        customparser.add_argument(
            "--getheaders",
            dest="getheaders",
            action="store_true",
            help="Use this flag to return the iLO response headers.",
            default=False,
        )
        customparser.add_argument(
            "--headers",
            dest="headers",
            help="Use this flag to add extra headers to the request."
            "\t\t\t\t\t Usage: --headers=HEADER:VALUE,HEADER:VALUE",
            default=None,
        )
        customparser.add_argument(
            "--silent",
            dest="silent",
            action="store_true",
            help="""Use this flag to silence responses""",
            default=False,
        )
        customparser.add_argument(
            "--service",
            dest="service",
            action="store_true",
            help="""Use this flag to enable service mode and increase the function speed""",
            default=False,
        )
        customparser.add_argument(
            "--expand",
            dest="expand",
            action="store_true",
            help="""Use this flag to expand the path specified using the """ """expand notation '?$expand=.'""",
            default=False,
        )



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
"""Command to show the pending configuration for PMM"""

from __future__ import absolute_import

try:
    from rdmc_helper import (
        LOGGER,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        ReturnCodes,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        LOGGER,
        NoContentsFoundForOperationError,
    )

from .lib.DisplayHelpers import DisplayHelpers, OutputFormats
from .lib.Mapper import Mapper
from .lib.MapperRenderers import MappingTable
from .lib.PmemHelpers import PmemHelpers
from .lib.RestHelpers import RestHelpers


class ShowPmemPendingConfigCommand:
    """
    Command to show the pending configuration for PMM
    """

    def __init__(self):
        self.ident = {
            "name": "showpmmpendingconfig",
            "usage": None,
            "description": "Shows the pending tasks for configuring PMM\n" "\texample: showpmmpendingconfig --json",
            "summary": "Shows the pending configuration for PMM.",
            "aliases": [],
            "auxcommands": [],
        }
        self._mapper = Mapper()
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()
        self._display_helpers = DisplayHelpers()
        self._pmem_helpers = PmemHelpers()

    def definearguments(self, customparser):
        """
        Wrapper function for new command main function
        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "-j",
            "--json",
            action="store_true",
            dest="json",
            help="Optionally include this flag to change the output to JSON format.",
            default=False,
        )

    def run(self, line, help_disp=False):
        """
        Wrapper function for new command main function
        :param line: command line input
        :type line: string.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        LOGGER.info("PMM Pending Configuration: %s", self.ident["name"])
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineError("Failed to parse options")
        if args:
            raise InvalidCommandLineError("Chosen command or flag doesn't expect additional arguments")
        # Raise exception if server is in POST
        if RestHelpers(rdmcObject=self.rdmc).in_post():
            raise NoContentsFoundForOperationError(
                "Unable to retrieve resources - " "server might be in POST or powered off"
            )
        self.show_pending_config(options)

        return ReturnCodes.SUCCESS

    def show_pending_config(self, options):
        """
        Command to display information about pending Persistent Memory Configuration
        :param options: command options
        :type options: options.
        """
        # retrieving task members
        task_members = RestHelpers(rdmcObject=self.rdmc).retrieve_task_members()

        # filtering task members
        filtered_task_members = RestHelpers(rdmcObject=self.rdmc).filter_task_members(task_members)
        if not filtered_task_members:
            self.rdmc.ui.printer("No pending configuration tasks found.\n\n")
            return None

        # retrieving memory members
        memory = RestHelpers(rdmcObject=self.rdmc).retrieve_memory_resources()
        if memory:
            memory_members = memory.get("Members")
        else:
            raise NoContentsFoundForOperationError("Failed to retrieve Memory Resources")

        attributes = ["Operation", "PmemSize", "VolatileSize", "DimmIds"]
        display_output = list()
        for task in filtered_task_members:
            # finding operation of task
            operation = self._mapper.get_single_attribute(task, "Operation", MappingTable.tasks.value, True)
            # displaying existing configuration for DELETE operation
            if operation.get("Operation", "") == "DELETE":
                target_uri = task.get("Payload").get("TargetUri")
                data = RestHelpers(rdmcObject=self.rdmc).get_resource(target_uri)
                table = MappingTable.delete_task.value
            else:
                task_type = self._mapper.get_single_attribute(task, "Type", MappingTable.tasks.value, True)
                task_type = task_type.get("Type", "")
                if task_type != "PMEM":
                    self.rdmc.ui.warn("Unsupported interleave set type found: " + task_type)
                    continue
                data = task
                table = MappingTable.tasks.value

            task_output = self._mapper.get_multiple_attributes(
                data,
                attributes,
                table,
                output_as_json=options.json,
                memory=memory_members,
            )
            display_output.append(task_output)

        if options.json:
            self._display_helpers.display_data(display_output, OutputFormats.json)
        else:
            self._display_helpers.display_data(display_output, OutputFormats.table)



# ##
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ##

# -*- coding: utf-8 -*-
""" Install Set Command for rdmc """

import json
import os

from six.moves import input

try:
    from rdmc_helper import (
        IncompatibleiLOVersionError,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        IncompatibleiLOVersionError,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )


class MakeInstallSetCommand:
    """Command class to create installset payload"""

    def __init__(self):
        self.ident = {
            "name": "makeinstallset",
            "usage": None,
            "description": "Run to enter a guided shell for making "
            "install sets. If not currently\n\tlogged into a server will perform "
            "basic guidance on making an installset,\n\tif logged into a server "
            "will provide guidance based on the current\n\tcomponents on the system. "
            "If you wish to use this command on a logged in\n\tserver upload the "
            "components before running for best results.",
            "summary": "Creates install sets for iLO.",
            "aliases": ["minstallset"],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()
        self.defaultprops = {
            "UpdatableBy": ["Bmc"],
            "Command": "ApplyUpdate",
            "WaitTimeSeconds": 0,
            "Filename": "",
        }
        self.helptext = {
            "Command": "Possible Commands: ApplyUpdate, ResetServer, " "ResetBmc, Wait",
            "UpdatableBy": "Possible Update parameter(s)"
            ":\nBmc: Updatable by iLO\nUefi: Updatable by Uefi\n"
            "RuntimeAgent: Updatable by runtime agent such as SUM/SUT",
            "WaitTimeSeconds": "Number of seconds to pause in Wait " "command.",
            "Filename": "Unique filename of component on " "iLO repository",
        }
        self.loggedin = None
        self.comps = None

    def run(self, line, help_disp=False):
        """Main installset worker function

        :param line: string of arguments passed in
        :type line: str.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.loggedin = None
        self.comps = None
        self.loggedin = self.minstallsetvalidation()

        if self.loggedin and self.rdmc.app.typepath.defs.isgen9:
            raise IncompatibleiLOVersionError("iLO Repository commands are " "only available on iLO 5.")

        self.rdmc.ui.warn("This command will run in interactive mode.\n")
        if args:
            raise InvalidCommandLineError("makeinstallset command takes no arguments.")

        self.minstallsetworker(options)

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def minstallsetworker(self, options):
        """Main installset creation worker

        :param options: command line options
        :type options: list.
        """
        comps = {}
        totcomps = []
        count = -1
        totcount = 0
        self.rdmc.ui.warn("Entering new shell, type quit to leave!\n")
        if self.loggedin:
            self.rdmc.ui.printer("Running in logged in mode.")
            self.comps = self.rdmc.app.getcollectionmembers("/redfish/v1/UpdateService/ComponentRepository/")
        else:
            self.rdmc.ui.printer("Running in basic guidance mode.")
        while True:
            comps = {}
            count = -1
            reqdprops = ["Command"]
            while True:
                if len(reqdprops) <= count:
                    break
                if count == -1:
                    line = input("\nEnter a name for this command: ")
                else:
                    self.rdmc.ui.printer("\n" + self.helptext[reqdprops[count]] + "\n")
                    if self.loggedin and reqdprops[count].lower() == "filename":
                        filenm, updateby, target = self.checkfiles(options)
                        comps["Filename"] = filenm
                        comps["UpdatableBy"] = updateby
                        if options.targets:
                            comps["Targets"] = target
                        break
                    else:
                        line = input("Enter " + reqdprops[count] + " for " + comps["Name"] + ": ")
                if line.endswith(os.linesep):
                    line.rstrip(os.linesep)
                if line == "quit":
                    break
                elif line == "" and not count == -1:
                    line = self.defaultprops[reqdprops[count]]
                if count == -1:
                    comps["Name"] = line
                else:
                    while True:
                        validated = self.validatepropvalue(reqdprops[count], line, reqdprops)
                        if not validated:
                            if line == "quit":
                                break
                            line = input(
                                "Input %s is not a valid property " "for %s. Try again: " % (line, reqdprops[count])
                            )
                        else:
                            comps[reqdprops[count]] = validated
                            break
                count = count + 1
            if line == "quit":
                break
            else:
                totcomps.append(comps)
                totcount = totcount + 1

        if not totcount:
            self.rdmc.ui.warn("No sequences created. Exiting without creating an installset.\n")
        else:
            while True:
                isrecovery = input("Is this a recovery installset? ")
                isrecovery = True if str(isrecovery).lower() in ["true", "t", "yes", "y"] else isrecovery
                isrecovery = False if str(isrecovery).lower() in ["false", "f", "no", "n"] else isrecovery
                if not isinstance(isrecovery, bool):
                    self.rdmc.ui.warn("'Isrecovery' should be either true or false.\n")
                    continue
                else:
                    break

            if isrecovery:
                installsetname = "System Recovery Set"
            else:
                while True:
                    installsetname = input("Enter installset name: ")
                    if not installsetname:
                        self.rdmc.ui.warn("Install set must have a name.\n")
                    else:
                        break

            description = input("Enter description for the installset: ")

            body = {
                "Name": installsetname,
                "Description": description,
                "IsRecovery": isrecovery,
                "Sequence": totcomps,
            }

            self.rdmc.ui.print_out_json(body)
            with open(options.filename, "w") as outfile:
                json.dump(body, outfile, indent=2, sort_keys=True)

            self.rdmc.ui.printer("installset saved to %s\n" % options.filename)

    def validatepropvalue(self, propvalue, givenvalue, reqdprops):
        """Validates a string returning the correct type

        :param propvalue: property to validate
        :type propvalue: string.
        :param givenvalue: value to validate
        :type givenvalue: string.
        :param reqdprops: the required property list
        :type reqdprops: list.
        """
        validated_property = None
        if propvalue == "WaitTimeSeconds":
            validated_property = int(givenvalue)
        elif propvalue == "UpdatableBy":
            if isinstance(givenvalue, list):
                validated_property = givenvalue
            value = [x.strip() for x in givenvalue.split(",")]
            for ind, item in enumerate(value):
                if item.lower() == "runtimeagent":
                    value[ind] = "RuntimeAgent"
                elif item.lower() == "uefi":
                    value[ind] = "Uefi"
                elif item.lower() == "bmc":
                    value[ind] = "Bmc"
            validated_property = value
        elif propvalue == "Command":
            if givenvalue.lower() == "applyupdate":
                if self.loggedin and not self.comps:
                    self.rdmc.ui.printer("All components on the system are already " "added to the installset.\n")
                else:
                    reqdprops.append("Filename")
                    reqdprops.append("UpdatableBy")
                    validated_property = "ApplyUpdate"
            elif givenvalue.lower() == "resetserver":
                validated_property = "ResetServer"
            elif givenvalue.lower() == "resetbmc":
                validated_property = "ResetBmc"
            elif givenvalue.lower() == "wait":
                reqdprops.append("WaitTimeSeconds")
                validated_property = "Wait"
        elif propvalue == "Filename":
            if givenvalue:
                validated_property = givenvalue

        return validated_property

    def get_target(self, target):
        target_list = target.split(",")
        for target in target_list:
            try:
                target_url = "/redfish/v1/UpdateService/FirmwareInventory/" + target
                dd = self.rdmc.app.get_handler(target_url, service=True, silent=True)
                if dd.status == 404:
                    return False
            except:
                return False
        return True

    def checkfiles(self, options):
        count = 0
        self.rdmc.ui.printer("Components currently in the repository that have not " "been added to the installset:\n")
        for comp in self.comps:
            count += 1
            self.rdmc.ui.printer("[%d] %s %s\n" % (count, comp["Name"], comp["Version"]))
        while True:
            userinput = input("Select the number of the component you want to add to " "the install set: ")
            try:
                userinput = int(userinput)
                if userinput > count or userinput == 0:
                    raise
                break
            except:
                self.rdmc.ui.warn("Input is not a valid number.\n")
        filename = self.comps[userinput - 1]["Filename"]
        updatableby = self.comps[userinput - 1]["UpdatableBy"]
        if options.targets:
            targets = []
            target_list = options.targets.split(",")
            for target in target_list:
                target_url = "/redfish/v1/UpdateService/FirmwareInventory/" + str(target) + "/"
                targets.append(target_url)
            verified = self.get_target(options.targets)
            if not verified:
                self.rdmc.ui.error("Provided target was not available, Please provide valid target id\n")
                return ReturnCodes.INVALID_TARGET_ERROR
            target = targets
            return filename, updatableby, target
        else:
            target = None
            del self.comps[userinput - 1]
            return filename, updatableby, target

    def minstallsetvalidation(self):
        """makeinstallset validation function"""

        try:
            _ = self.rdmc.app.current_client
            loggedin = True
        except:
            loggedin = False

        return loggedin

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "-f",
            "--filename",
            dest="filename",
            help="Use this flag if you wish to use a different"
            " filename than the default one. The default filename is"
            " myinstallset.json",
            default="myinstallset.json",
        )
        customparser.add_argument(
            "--targets",
            help="If targets value specify a comma separated" "firmwareinventory id only",
            metavar="targets_indices",
        )



"""
conftest.py is used to set up shared fixtures, plugins, hooks, other config etc.
This is the conftest.py for integration tests
http://docs.pytest.org/en/latest/writing_plugins.html#conftest-py-local-per-directory-plugins
"""
from __future__ import absolute_import

import logging
import re
import sys

import pytest

from ..functional.conftest import RestClient, ilorest, test_server
from .ilocli import IloCli
from .ilodebug import IloDebugger

ilo_debug_logger = logging.getLogger('ilodebug')
vsp_logger = logging.getLogger('vsp')


def pytest_configure(config):
    # configure logging
    formatter = logging.Formatter('%(asctime)s [%(levelname)s] %(message)s')

    ilo_debug_file = config.getoption('--ilodebug')
    if ilo_debug_file:
        sys.stdout.write("logging iLO debugger output to: {}".format(ilo_debug_file))
        ilo_debug_logger.setLevel(logging.DEBUG)
        fh = logging.FileHandler(ilo_debug_file, mode='wb+')
        fh.setLevel(logging.DEBUG)
        fh.setFormatter(formatter)
        ilo_debug_logger.addHandler(fh)

    vsp_file = config.getoption('--vsp')
    if vsp_file:
        sys.stdout.write("logging iLO Virtual Serial Port output to: {}".format(vsp_file))
        vsp_logger.setLevel(logging.DEBUG)
        fh = logging.FileHandler(vsp_file, mode='wb+')
        fh.setLevel(logging.DEBUG)
        fh.setFormatter(formatter)
        vsp_logger.addHandler(fh)


def pytest_runtest_makereport(item, call):
    # incremental testing marker
    if "incremental" in item.keywords:
        if call.excinfo is not None:
            parent = item.parent
            parent._previousfailed = item


def pytest_runtest_setup(item):
    # incremental testing marker
    if "incremental" in item.keywords:
        previousfailed = getattr(item.parent, "_previousfailed", None)
        if previousfailed is not None:
            pytest.xfail("previous test failed (%s)" % previousfailed.name)


@pytest.fixture(scope='session', autouse=True)
def clear_nand_counters(test_server, request):
    """Clear all iLO NAND write counters using the iLO debugger."""
    if request.config.getoption('--clear-nand-counters'):
        with IloDebugger(test_server.hostname) as ilo_debugger:
            sys.stdout.write("Clearing all iLO NAND write counters (setting timers to max)")
            output = ilo_debugger.run("otmax all")
            assert "Set daily timer to MAX value" in output, \
                "Failed to clear NAND write counters:\n {}".format(output)


@pytest.fixture
def ilodebug(test_server, request):
    """Start an iLO debugger session and log all output."""
    if request.config.getoption('--ilodebug'):
        test_path = request.node.nodeid.replace('::()::', '::')
        ilo_debug_logger.info(test_path)

        with IloDebugger(test_server.hostname) as ilo_debugger:
            # get current system time
            output = ilo_debugger.run('id')
            ilo_debug_logger.debug(output)
            yield ilo_debugger
            output = ilo_debugger.read()
            if output:
                ilo_debug_logger.debug('\n' + output)
    else:
        yield


@pytest.fixture
def vsp(test_server, request, set_uefi_debug):
    """Start an iLO Virtual Serial Port session and log all output."""
    if request.config.getoption('--vsp'):
        test_path = request.node.nodeid.replace('::()::', '::')
        vsp_logger.info(test_path)

        hostname = test_server.hostname
        username, password = test_server.username, test_server.password

        with IloCli(hostname, username, password) as cli:
            if cli is None:
                yield
            else:
                # get current system time
                cli.sendline("time")
                with cli.vsp_session():
                    yield cli
                output = cli.read()
                if output:
                    vsp_logger.debug('\n' + output)
    else:
        yield


@pytest.fixture
def set_uefi_debug(test_server, request):
    """Temporarily set the UEFI serial debug level using the iLO debugger.

    The debug levels are: 0 (Disabled), 1 (ErrorsOnly), 2 (Medium), 3 (Verbose).
    """
    new_dbg = request.config.getoption('--uefi-debug-level')
    if new_dbg is not None:
        with IloDebugger(test_server.hostname) as debugger:
            # save the current debug level
            output = debugger.run("evs name cqhdbg")
            match = re.search('EVS NAME "CQHDBG".+\n.+([0-9]{2})', output)
            current_dbg = "0x{}".format(match.group(1)) if match else None

            sys.stdout.write("setting UEFI serial debug level to {} (current {})".format(new_dbg, current_dbg))
            debugger.run("evs xset cqhdbg {}".format(new_dbg))

        yield
        # restore previous debug level
        if current_dbg:
            with IloDebugger(test_server.hostname) as debugger:
                debugger.run("evs xset cqhdbg {}".format(current_dbg))
    else:
        yield



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" Server State Command for rdmc """

try:
    from rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ReturnCodes,
    )


class ServerStateCommand:
    """Returns the current state of the server that  is currently logged in"""

    def __init__(self):
        self.ident = {
            "name": "serverstate",
            "usage": None,
            "description": "Returns the current state of the" " server\n\n\tExample: serverstate",
            "summary": "Returns the current state of the server.",
            "aliases": [],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main serverstate function

        :param line: string of arguments passed in
        :type line: str.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        if args:
            raise InvalidCommandLineError(
                "Invalid number of parameters, " "serverstate command does not take any parameters."
            )

        self.serverstatevalidation(options)

        path = self.rdmc.app.typepath.defs.systempath
        results = self.rdmc.app.get_handler(path, silent=True, uncache=True).dict

        if results:
            results = results["Oem"][self.rdmc.app.typepath.defs.oemhp]["PostState"]
            self.rdmc.ui.printer("The server is currently in state: " + results + "\n")
        else:
            raise NoContentsFoundForOperationError("Unable to retrieve server state")

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def serverstatevalidation(self, options):
        """Server state method validation function

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)
